import dataclasses


@dataclasses.dataclass
class AiscRectangular(object):
    """This is a dataclass containing the properties of an AISC Rectangular section, typically representing
    rectangular HSS (Hollow Structural Section) properties. Properties follow the AISC shapes database.

    :param A: Cross-sectional area (in^2)
    :type A: float
    :param AISC_name: The name of the AISC section
    :type AISC_name: str
    :param Bout: Overall (outer) width of the rectangular HSS (in)
    :type Bout: float
    :param C: HSS torsional constant (in^3)
    :type C: float
    :param EDI_Std_Nomenclature: The EDI standard nomenclature name
    :type EDI_Std_Nomenclature: str
    :param Ht: Overall (outer) depth of the longer wall of rectangular HSS (in)
    :type Ht: float
    :param Ix: Moment of inertia about the x-axis (in^4)
    :type Ix: float
    :param Iy: Moment of inertia about the y-axis (in^4)
    :type Iy: float
    :param J: Torsional constant (in^4)
    :type J: float
    :param Sx: Elastic section modulus about the x-axis (in^3)
    :type Sx: float
    :param Sy: Elastic section modulus about the y-axis (in^3)
    :type Sy: float
    :param T_F: Whether the section has an additional note in the AISC shapes database (T or F)
    :type T_F: str
    :param Type: The section type
    :type Type: str
    :param W: Nominal weight (lb/ft)
    :type W: float
    :param Zx: Plastic section modulus about the x-axis (in^3)
    :type Zx: float
    :param Zy: Plastic section modulus about the y-axis (in^3)
    :type Zy: float
    :param bin: Width of the shorter flat wall of rectangular HSS (in)
    :type bin: float
    :param b_tdes: Slenderness ratio for the shorter wall of rectangular HSS, b/tdes
    :type b_tdes: float
    :param h: Depth of the longer flat wall of rectangular HSS (in)
    :type h: float
    :param h_tdes: Slenderness ratio for the longer wall of rectangular HSS, h/tdes
    :type h_tdes: float
    :param rx: Radius of gyration about the x-axis (in)
    :type rx: float
    :param ry: Radius of gyration about the y-axis (in)
    :type ry: float
    :param tdes: Design thickness of HSS wall (in)
    :type tdes: float
    :param tnom: Nominal thickness of HSS wall (in)
    :type tnom: float
    """

    A: float
    AISC_name: str
    Bout: float
    C: float
    EDI_Std_Nomenclature: str
    Ht: float
    Ix: float
    Iy: float
    J: float
    Sx: float
    Sy: float
    T_F: str
    Type: str
    W: float
    Zx: float
    Zy: float
    bin: float
    b_tdes: float
    h: float
    h_tdes: float
    rx: float
    ry: float
    tdes: float
    tnom: float


ALL_AISC_RECTANGULAR_NAMES = (
    "HSS24X12X1/2",
    "HSS24X12X3/4",
    "HSS20X12X5/16",
    "HSS20X8X5/8",
    "HSS24X12X5/8",
    "HSS20X8X3/8",
    "HSS20X8X1/2",
    "HSS20X4X1/2",
    "HSS20X8X5/16",
    "HSS20X4X3/8",
    "HSS20X4X5/16",
    "HSS20X4X1/4",
    "HSS20X20X5/8",
    "HSS22X22X7/8",
    "HSS22X22X3/4",
    "HSS20X20X7/8",
    "HSS20X20X3/4",
    "HSS18X18X3/4",
    "HSS20X12X3/8",
    "HSS20X12X1/2",
    "HSS18X18X7/8",
    "HSS20X20X1/2",
    "HSS20X12X5/8",
    "HSS20X12X3/4",
    "HSS18X6X5/8",
    "HSS18X6X3/8",
    "HSS18X6X1/2",
    "HSS18X18X5/8",
    "HSS18X6X5/16",
    "HSS18X18X1/2",
    "HSS18X6X1/4",
    "HSS16X16X3/4",
    "HSS16X16X5/8",
    "HSS16X16X1/2",
    "HSS16X16X3/8",
    "HSS16X16X5/16",
    "HSS16X12X3/4",
    "HSS16X12X5/8",
    "HSS16X12X1/2",
    "HSS16X12X3/8",
    "HSS16X12X5/16",
    "HSS16X8X5/8",
    "HSS16X8X1/2",
    "HSS16X8X3/8",
    "HSS16X8X5/16",
    "HSS16X8X1/4",
    "HSS16X4X5/8",
    "HSS16X4X1/2",
    "HSS16X4X3/8",
    "HSS16X4X5/16",
    "HSS16X16X7/8",
    "HSS16X4X1/4",
    "HSS16X4X3/16",
    "HSS14X14X7/8",
    "HSS14X14X3/4",
    "HSS14X14X5/8",
    "HSS14X14X1/2",
    "HSS14X14X3/8",
    "HSS14X14X5/16",
    "HSS14X10X5/8",
    "HSS14X10X1/2",
    "HSS14X10X3/8",
    "HSS14X6X5/8",
    "HSS14X10X5/16",
    "HSS14X10X1/4",
    "HSS14X6X5/16",
    "HSS14X6X1/2",
    "HSS14X6X3/8",
    "HSS14X4X5/8",
    "HSS14X6X1/4",
    "HSS14X6X3/16",
    "HSS14X4X5/16",
    "HSS14X4X3/16",
    "HSS14X4X1/4",
    "HSS14X4X1/2",
    "HSS12X12X5/8",
    "HSS14X4X3/8",
    "HSS12X12X3/4",
    "HSS12X12X1/2",
    "HSS12X12X5/16",
    "HSS12X10X1/2",
    "HSS12X10X5/16",
    "HSS12X10X3/8",
    "HSS12X12X1/4",
    "HSS12X12X3/16",
    "HSS12X12X3/8",
    "HSS12X8X1/2",
    "HSS12X8X5/8",
    "HSS12X10X1/4",
    "HSS12X8X5/16",
    "HSS12X8X1/4",
    "HSS12X8X3/8",
    "HSS12X6X5/8",
    "HSS12X8X3/16",
    "HSS12X6X1/2",
    "HSS12X6X3/8",
    "HSS12X6X5/16",
    "HSS12X6X1/4",
    "HSS12X6X3/16",
    "HSS12X4X5/8",
    "HSS12X4X1/2",
    "HSS12X4X3/8",
    "HSS12X4X5/16",
    "HSS12X4X1/4",
    "HSS12X4X3/16",
    "HSS12X3-1/2X3/8",
    "HSS12X3-1/2X5/16",
    "HSS12X3X5/16",
    "HSS12X3X1/4",
    "HSS12X3X3/16",
    "HSS12X2X1/4",
    "HSS12X2X5/16",
    "HSS12X2X3/16",
    "HSS10X10X3/4",
    "HSS10X10X5/8",
    "HSS10X10X1/2",
    "HSS10X10X3/8",
    "HSS10X10X1/4",
    "HSS10X10X5/16",
    "HSS10X10X3/16",
    "HSS10X8X5/8",
    "HSS10X8X1/2",
    "HSS10X8X3/8",
    "HSS10X8X5/16",
    "HSS10X8X1/4",
    "HSS10X8X3/16",
    "HSS10X6X5/8",
    "HSS10X6X1/2",
    "HSS10X6X3/8",
    "HSS10X6X5/16",
    "HSS10X6X1/4",
    "HSS10X5X3/8",
    "HSS10X6X3/16",
    "HSS10X5X1/4",
    "HSS10X5X5/16",
    "HSS10X4X5/8",
    "HSS10X5X3/16",
    "HSS10X4X3/8",
    "HSS10X4X1/2",
    "HSS10X4X1/4",
    "HSS10X4X5/16",
    "HSS10X4X3/16",
    "HSS10X3-1/2X1/2",
    "HSS10X3-1/2X3/8",
    "HSS10X4X1/8",
    "HSS10X3-1/2X5/16",
    "HSS10X3-1/2X3/16",
    "HSS10X3-1/2X1/4",
    "HSS10X3-1/2X1/8",
    "HSS10X3X3/8",
    "HSS10X3X1/4",
    "HSS10X3X5/16",
    "HSS10X3X3/16",
    "HSS10X3X1/8",
    "HSS10X2X3/8",
    "HSS10X2X5/16",
    "HSS10X2X1/4",
    "HSS10X2X3/16",
    "HSS10X2X1/8",
    "HSS9X9X5/8",
    "HSS9X9X1/2",
    "HSS9X9X3/8",
    "HSS9X9X3/16",
    "HSS9X9X5/16",
    "HSS9X9X1/4",
    "HSS9X7X3/8",
    "HSS9X7X1/2",
    "HSS9X7X5/8",
    "HSS9X9X1/8",
    "HSS9X7X3/16",
    "HSS9X7X5/16",
    "HSS9X7X1/4",
    "HSS9X5X3/8",
    "HSS9X5X5/16",
    "HSS9X5X1/2",
    "HSS9X5X1/4",
    "HSS9X5X3/16",
    "HSS9X3X1/2",
    "HSS9X3X3/8",
    "HSS9X3X5/16",
    "HSS9X3X1/4",
    "HSS9X3X3/16",
    "HSS8X8X5/8",
    "HSS8X8X1/2",
    "HSS8X8X3/8",
    "HSS8X8X5/16",
    "HSS8X8X1/4",
    "HSS8X8X3/16",
    "HSS8X8X1/8",
    "HSS8X6X5/8",
    "HSS9X5X5/8",
    "HSS8X6X1/2",
    "HSS8X6X3/8",
    "HSS8X6X5/16",
    "HSS8X6X1/4",
    "HSS8X6X3/16",
    "HSS8X4X5/8",
    "HSS8X4X1/2",
    "HSS8X4X5/16",
    "HSS8X4X3/8",
    "HSS8X4X1/8",
    "HSS8X4X1/4",
    "HSS8X4X3/16",
    "HSS8X3X1/4",
    "HSS8X3X1/2",
    "HSS8X3X3/8",
    "HSS7X7X5/16",
    "HSS7X7X1/4",
    "HSS7X7X3/16",
    "HSS7X7X1/8",
    "HSS7X5X1/2",
    "HSS7X5X3/8",
    "HSS7X5X5/16",
    "HSS7X5X1/4",
    "HSS7X5X3/16",
    "HSS7X5X1/8",
    "HSS7X4X1/2",
    "HSS8X2X1/4",
    "HSS7X4X3/8",
    "HSS7X4X1/4",
    "HSS7X4X5/16",
    "HSS7X4X3/16",
    "HSS7X4X1/8",
    "HSS8X3X3/16",
    "HSS7X3X1/2",
    "HSS8X3X1/8",
    "HSS8X2X5/16",
    "HSS8X2X3/8",
    "HSS8X3X5/16",
    "HSS7X7X5/8",
    "HSS8X2X1/8",
    "HSS8X2X3/16",
    "HSS7X3X3/8",
    "HSS7X3X5/16",
    "HSS7X7X1/2",
    "HSS7X3X1/4",
    "HSS7X3X1/8",
    "HSS7X3X3/16",
    "HSS7X2X1/8",
    "HSS7X2X3/16",
    "HSS6X6X1/2",
    "HSS7X2X1/4",
    "HSS6X6X3/8",
    "HSS6X6X5/8",
    "HSS6X6X5/16",
    "HSS6X6X1/4",
    "HSS6X6X3/16",
    "HSS7X7X3/8",
    "HSS6X6X1/8",
    "HSS6X5X1/2",
    "HSS6X5X3/8",
    "HSS6X5X5/16",
    "HSS6X5X1/4",
    "HSS6X5X3/16",
    "HSS6X5X1/8",
    "HSS6X4X5/16",
    "HSS6X4X1/8",
    "HSS6X4X1/4",
    "HSS6X4X3/16",
    "HSS6X3X1/4",
    "HSS6X3X3/8",
    "HSS6X4X1/2",
    "HSS6X3X1/8",
    "HSS6X3X3/16",
    "HSS6X2X3/8",
    "HSS6X2X5/16",
    "HSS5-1/2X5-1/2X3/8",
    "HSS5X5X3/8",
    "HSS6X3X5/16",
    "HSS5-1/2X5-1/2X1/4",
    "HSS6X4X3/8",
    "HSS6X3X1/2",
    "HSS5X5X5/16",
    "HSS6X2X3/16",
    "HSS5-1/2X5-1/2X5/16",
    "HSS6X2X1/8",
    "HSS5-1/2X5-1/2X3/16",
    "HSS6X2X1/4",
    "HSS5-1/2X5-1/2X1/8",
    "HSS5X5X1/4",
    "HSS5X5X1/8",
    "HSS5X5X1/2",
    "HSS5X4X1/2",
    "HSS5X4X3/8",
    "HSS5X4X5/16",
    "HSS5X4X3/16",
    "HSS5X4X1/4",
    "HSS5X5X3/16",
    "HSS5X4X1/8",
    "HSS5X3X3/8",
    "HSS5X3X1/2",
    "HSS5X3X3/16",
    "HSS5X2-1/2X1/4",
    "HSS5X3X5/16",
    "HSS5X3X1/8",
    "HSS5X2X3/8",
    "HSS5X3X1/4",
    "HSS5X2X1/8",
    "HSS5X2X3/16",
    "HSS5X2X1/4",
    "HSS5X2-1/2X1/8",
    "HSS5X2-1/2X3/16",
    "HSS5X2X5/16",
    "HSS4X4X1/8",
    "HSS4-1/2X4-1/2X5/16",
    "HSS4-1/2X4-1/2X3/16",
    "HSS4X3X3/8",
    "HSS4X3X5/16",
    "HSS4-1/2X4-1/2X3/8",
    "HSS4X3X3/16",
    "HSS4-1/2X4-1/2X1/4",
    "HSS4X3X1/4",
    "HSS4X3X1/8",
    "HSS4X2-1/2X3/8",
    "HSS4X2-1/2X5/16",
    "HSS4-1/2X4-1/2X1/2",
    "HSS4X4X3/8",
    "HSS4-1/2X4-1/2X1/8",
    "HSS4X4X5/16",
    "HSS4X4X1/2",
    "HSS4X4X3/16",
    "HSS4X4X1/4",
    "HSS4X2-1/2X1/4",
    "HSS4X2-1/2X3/16",
    "HSS4X2-1/2X1/8",
    "HSS4X2X3/8",
    "HSS4X2X1/4",
    "HSS4X2X5/16",
    "HSS4X2X1/8",
    "HSS3-1/2X3-1/2X3/8",
    "HSS3-1/2X3-1/2X5/16",
    "HSS3-1/2X3-1/2X1/8",
    "HSS4X2X3/16",
    "HSS3-1/2X3-1/2X3/16",
    "HSS3-1/2X3-1/2X1/4",
    "HSS3-1/2X2-1/2X3/8",
    "HSS3-1/2X2-1/2X5/16",
    "HSS3-1/2X2-1/2X1/4",
    "HSS3-1/2X2-1/2X3/16",
    "HSS3-1/2X2-1/2X1/8",
    "HSS3-1/2X2X3/16",
    "HSS3-1/2X2X1/4",
    "HSS3-1/2X1-1/2X1/4",
    "HSS3-1/2X2X1/8",
    "HSS3-1/2X1-1/2X3/16",
    "HSS3-1/2X1-1/2X1/8",
    "HSS3X3X3/8",
    "HSS3X3X5/16",
    "HSS3X3X1/4",
    "HSS3X3X3/16",
    "HSS3X3X1/8",
    "HSS3X2-1/2X5/16",
    "HSS3X2-1/2X1/4",
    "HSS3X2-1/2X3/16",
    "HSS3X2-1/2X1/8",
    "HSS3X2X5/16",
    "HSS3X2X1/4",
    "HSS3X2X3/16",
    "HSS3X2X1/8",
    "HSS3X1-1/2X1/4",
    "HSS3X1-1/2X3/16",
    "HSS3X1-1/2X1/8",
    "HSS3X1X1/8",
    "HSS3X1X3/16",
    "HSS2-1/2X2-1/2X1/4",
    "HSS2-1/2X2-1/2X5/16",
    "HSS2-1/2X2-1/2X3/16",
    "HSS2-1/2X2-1/2X1/8",
    "HSS2-1/2X2X1/8",
    "HSS2-1/2X2X1/4",
    "HSS2-1/2X2X3/16",
    "HSS2-1/2X1-1/2X1/8",
    "HSS2-1/2X1X3/16",
    "HSS2-1/2X1-1/2X1/4",
    "HSS2-1/4X2-1/4X1/4",
    "HSS2-1/2X1X1/8",
    "HSS2-1/2X1-1/2X3/16",
    "HSS2-1/4X2-1/4X1/8",
    "HSS2-1/4X2X3/16",
    "HSS2-1/4X2-1/4X3/16",
    "HSS2-1/4X2X1/8",
    "HSS2X2X3/16",
    "HSS2X2X1/4",
    "HSS2X1-1/2X3/16",
    "HSS2X2X1/8",
    "HSS2X1-1/2X1/8",
    "HSS2X1X1/8",
    "HSS2X1X3/16",
    "HSS6X2X14Ga",
    "HSS2X2X14Ga",
    "HSS4X2X14Ga",
)
